use strict;

package FileTimeTest;

use base qw(TWikiTestCase);

use TWiki::Contrib::DBCacheContrib::Archive;
use TWiki::Contrib::DBCacheContrib::FileTime;
use TWiki::Contrib::DBCacheContrib::Array;
use TWiki::Func;
use Storable;

my $files; # fixture
my $web = "FT"; # fixture
my $root;
my $acache;
my $scache;
my $twiki;

sub new {
  my $self = shift()->SUPER::new(@_);
  return $self;
}

sub _readFile {
    my $f = shift;
    local $/ = undef;
    open(F,"<",$f) || die $!;
    my $x = <F>;
    close(F);
    return $x;
}

sub set_up {
  my $this = shift;

  $this->SUPER::set_up();

  $twiki = new TWiki( "TestUser1" );

  my $dbt = _readFile("./testDB.dat");
  $root = TWiki::Func::getDataDir() . "/$web";
  $files = new TWiki::Contrib::DBCacheContrib::Array();
  foreach my $t ( split(/\<TOPIC\>/,$dbt)) {
    if ( $t =~ m/\"(.*?)\"/o ) {
      $twiki->{store}->saveTopic( $twiki->{user}, $web, $1, $t, undef);
      $files->add(new TWiki::Contrib::DBCacheContrib::FileTime( "$root/$1.txt" ));
    }
  }
  $acache = "$root/cache.Archive";
  $scache = "$root/cache.Storable";
}

sub test_OK {
  my $this = shift;
  # Make sure the file times reflect what's on disc
  foreach my $ft ( $files->getValues() ) {
    $this->assert($ft->uptodate());
  }
}

sub test_touchOne {
  my $this = shift;
  sleep(1);# make sure file times are different
  `touch $root/Dir4.txt`;

  foreach my $ft ( $files->getValues() ) {
    if ( $ft->{file} eq "$root/Dir4.txt") {
      $this->assert(!$ft->uptodate());
    } else {
      $this->assert($ft->uptodate());
    }
  }
}

sub test_delOne {
  my $this = shift;
  `rm $root/Dir2.txt`;

  foreach my $ft ( $files->getValues() ) {
    if ( $ft->{file} eq "$root/Dir2.txt") {
      $this->assert(!$ft->uptodate());
    } else {
      $this->assert($ft->uptodate());
    }
  }
}

1;
